//===================================================================
//                                                                  =
// Author:    Martin Berriman                                       =
// Copyright: Innovatronics Ltd 2006                                =
//                                                                  =
//===================================================================

#ifndef _EEPROMC
#define _EEPROMC

// HEADERS ==========================================================
#include "eeprom.h"
// END HEADERS ======================================================

// C FILES ==========================================================
// END C FILES ======================================================

// MACROS =========================================================
// END MACROS =====================================================

// GLOBALS ==========================================================
//reserve some space for the config data
//#define CONFIG_DATA_START_ADDR getenv("PROGRAM_MEMORY") - 64
#define CONFIG_DATA_START_ADDR 0xBFBF //0xBFFF - 64 bytes
//#org CONFIG_DATA_START_ADDR, CONFIG_DATA_START_ADDR + 63 {}
//reserve some more space for the custom data
//#define CONFIG_CUSTOM_START_ADDR CONFIG_DATA_START_ADDR - 64
#define CONFIG_CUSTOM_START_ADDR 0xBF7F //CONFIG_DATA_START_ADDR - 64 bytes
//#org CONFIG_CUSTOM_START_ADDR, CONFIG_CUSTOM_START_ADDR + 63 {}

// Put some sample data in ROM.
//Load default values into eeprom when mcu is programmed
#rom int8 CONFIG_DATA_START_ADDR = {"14Apr06",								//Cal_Date
									0x01, 0x00,																	//serial_no
									0x00,																				//units
									0x00,																				//standard_number
									0x00,0x00,																	//offset0
									0x00,0x00,																	//offset1
									0xA0,0x86,0x01,0x00,												//slope0 - read back to front
									0xA0,0x86,0x01,0x00}												//slope1 - read back to front
//Load default values into eeprom when mcu is programmed
#rom int8 CONFIG_CUSTOM_START_ADDR = {"CUSTOM\0     ",				//name
																						0x01, 0x00,				//max_graph_pa;
																						0x01, 0x00,				//min_graph_pa;
																						0x01, 0x00,				//max_graph_mbar;
																						0x01, 0x00,				//min_graph_mbar;
																						0x01, 0x00,				//max_graph_mmh2o;
																						0x01, 0x00,				//min_graph_mmh2o;
																						0x01, 0x00,				//max_graph_inh2o;
																						0x01, 0x00,				//min_graph_inh2o;
																						0x01, 0x00,				//limit;
																						0x01, 0x00,				//total_samples;
																						0x01, 0x00,				//fast_fill_pa;
																						0x01, 0x00,				//slow_fill_pa;
																						0x01, 0x00,				//stabilise_pa;
																						0x01, 0x00,				//test_pressure_pa;
																						0x01, 0x00,				//stabilise_time_seconds;
																						0x00,							//stabilise_hysteresis_pa;
																						0x00,							//stabilise_dwell_seconds;
																						0x00}							//test_deflate_pause_seconds;
// END GLOBALS ======================================================

		
//----------------------------------------------------------------------------------
void read_eeprom_custom_params(STANDARD_TYPE * test_standard_info)
//----------------------------------------------------------------------------------
{
	read_program_memory(CONFIG_CUSTOM_START_ADDR, test_standard_info, sizeof(STANDARD_TYPE));
}

//----------------------------------------------------------------------------------
void write_eeprom_custom_params(STANDARD_TYPE * test_standard_info)
//----------------------------------------------------------------------------------
{
	write_program_memory(CONFIG_CUSTOM_START_ADDR, test_standard_info, sizeof(STANDARD_TYPE));
}

//----------------------------------------------------------------------------------
void read_eeprom_data(void)
//----------------------------------------------------------------------------------
{
	EEPROM_DATA_TYPE eeprom_data;

	read_program_memory(CONFIG_DATA_START_ADDR, &eeprom_data, sizeof(EEPROM_DATA_TYPE));

	strncpy(&cal_date[0], eeprom_data.cal_date, 8);
	serial_no = eeprom_data.serial_no;
	units = eeprom_data.units;
	test_standard_number = eeprom_data.standard_number;
	cal_offset[0] = eeprom_data.cal_offset0;
	cal_offset[1] = eeprom_data.cal_offset1;
	cal_slope[0]  = eeprom_data.cal_slope0;
	cal_slope[1]  = eeprom_data.cal_slope1;
}

//----------------------------------------------------------------------------------
void write_eeprom_data(void)
//----------------------------------------------------------------------------------
{
	EEPROM_DATA_TYPE eeprom_data;
	unsigned int32	ee_block_address;	// Contains the address of the first byte of the block
	unsigned int16 * ptr;

	ee_block_address = CONFIG_DATA_START_ADDR;
	ptr = &eeprom_data;

	strncpy(eeprom_data.cal_date, &cal_date[0], 8);
	eeprom_data.serial_no = serial_no;
	eeprom_data.units = units;
	eeprom_data.standard_number = test_standard_number;
	eeprom_data.cal_offset0 = cal_offset[0];
	eeprom_data.cal_offset1 = cal_offset[1];
	eeprom_data.cal_slope0 = cal_slope[0];
	eeprom_data.cal_slope1 = cal_slope[1];

	// NOTE: Program EEPROM is automatically erased when full blocks are written, otherwise it is not
	write_program_memory(ee_block_address, ptr, 64);
}

// =============================================================
#endif
