//===================================================================
//                                                                  =
// Author:    Martin Berriman                                       =
// Copyright: Innovatronics Ltd 2006                                =
//                                                                  =
//===================================================================

#ifndef _RS232C
#define _RS232C

// HEADERS ==========================================================
#include "rs232.h"
// END HEADERS ======================================================

// C FILES ==========================================================
// END C FILES ======================================================

// MACROS =========================================================
#define bkbhit (next_in != next_out)
// END MACROS =====================================================

// GLOBALS ==========================================================
static unsigned char rx_str   [BUFFER_SIZE];
static unsigned char rx_buffer[BUFFER_SIZE];
int8 next_in = 0;
int8 next_out = 0;
// END GLOBALS ======================================================

//----------------------------------------------------------------------------------
#int_rda  //Interrupt driven RS232 routine
void rx232_isr(void)
//----------------------------------------------------------------------------------
{
   int8 t;

   rx_buffer[next_in] = getc();
   rx_buffer[next_in] = rx_buffer[next_in];
   t = next_in;
   next_in = (next_in + 1) % BUFFER_SIZE;

   if (next_in == next_out)
   {
      next_in=t;           // Buffer full !!
   }
}

//----------------------------------------------------------------------------------
unsigned char bgetc()
//----------------------------------------------------------------------------------
{
   unsigned char c;

   //If there is no data then we must wait for some
   while(!bkbhit)
      {
      };

   c = rx_buffer[next_out];
   next_out = (next_out + 1) % BUFFER_SIZE;
   return c;
}

//----------------------------------------------------------------------------------
void bgets(unsigned char * str, int8 maxLen)
//----------------------------------------------------------------------------------
{
   while (bkbhit && maxLen > 0)
   {
      *str++ = bgetc();
      maxLen--;
   }

   *str = '\0';
}

//----------------------------------------------------------------------------------
void rs232_write_bytes(unsigned char * str, int8 strLen)
//----------------------------------------------------------------------------------
{
   while (strLen > 0)
   {
      putc(*str++);
      strLen--;
   }
}

//----------------------------------------------------------------------------------
unsigned int8 check_command()
//----------------------------------------------------------------------------------
{
	signed int8 command = CMD_NOOP;

	if (bkbhit)
	{
		delay_ms(50); //Allow rest of message to arrive
		bgets(&rx_str[0], BUFFER_SIZE);
		command = (unsigned int8) rx_str[0];
	}

	return command;
}

// =============================================================
#endif