//===================================================================
//                                                                  =
// Author:    Martin Berriman                                       =
// Copyright: Innovatronics Ltd 2006                                =
//                                                                  =
//===================================================================

#ifndef _TESTINGC
#define _TESTINGC

// HEADERS ==========================================================
#include "testing.h"
// END HEADERS ======================================================

// C FILES ==========================================================
// END C FILES ======================================================

// MACROS =========================================================
// END MACROS =====================================================

// GLOBALS ==========================================================
enum test_states {state_idle, state_inflate_fast, state_inflate_slow, state_stabilising, state_prep_test, state_testing, state_complete, state_begin_test, state_calibrating}; 
static test_states test_state[2]; 
static int8 dwell_time[2];
static int16 final_pressure[2];
static int16 inflate_duration[2];
enum solenoids {sol_fast1, sol_slow1, sol_vent1, sol_fast2, sol_slow2, sol_vent2};
// END GLOBALS ======================================================

//----------------------------------------------------------------------------------
void start_testing(void)
//----------------------------------------------------------------------------------
{
	if (!testing)
	{
		initialise_test(0);
		reset_solenoids(0);

		if (dual_channel)
		{
			initialise_test(1);
			reset_solenoids(1);
		}
		testing = true;
		update_test_status();
	}
}

//----------------------------------------------------------------------------------
void initialise_test(unsigned int8 channel)
//----------------------------------------------------------------------------------
{
	if (channel == 0)
	{
		stabilise[0] = test_standard_info.stabilise_time_seconds;
		test_state[0] = state_begin_test;
		final_pressure[0] = 0;
		sample_num[0] = 1;
		inflate_duration[0] = 0;
		test_result1 = false;
	}
	else
	{
		stabilise[1] = test_standard_info.stabilise_time_seconds;
		test_state[1] = state_begin_test;
		final_pressure[1] = 0;
		sample_num[1] = 1;
		inflate_duration[1] = 0;
		test_result2 = false;
	}
}

//----------------------------------------------------------------------------------
void update_test_status(void)
//----------------------------------------------------------------------------------
{
	unsigned int16 mins;

	if (!testing)
	{
		if (test_state[0] == state_idle)
		{
			load_string_std_name(lcdStr);
			lcd_printf(1, 2, lcdStr);
			load_string_caldate(lcdStr, cal_date);
			lcd_printf(1, 3, lcdStr);
			load_string_idle(lcdStr);
			lcd_printf(1, 4, lcdStr);
		}
	}
	else
	{
		if (!dual_channel)
		{
			read_single_pressure(&pressure[0]);
			check_test_state();
			lcd_clear_line(2);

			switch (test_state[0])
			{
				case state_inflate_fast:
					load_string_test_info(lcdStr, st_test_single_inflate_fast);
					lcd_printf(1, 3, lcdStr);
					load_string_test_info(lcdStr, st_test_single_pressure);
					lcd_printf(1, 4, lcdStr);
					break;				
				case state_inflate_slow:
					load_string_test_info(lcdStr, st_test_single_inflate_slow);
					lcd_printf(1, 3, lcdStr);
					load_string_test_info(lcdStr, st_test_single_pressure);
					lcd_printf(1, 4, lcdStr);
					break;				
				case state_stabilising:
					load_string_test_info(lcdStr, st_test_single_stabilising);
					mins = stabilise[0] / 60; 
					fix_string_long(lcdStr, mins, 2, false);
					fix_string_long(lcdStr, stabilise[0] - (mins * 60), 2, false);
					lcd_printf(1, 3, lcdStr);
					load_string_test_info(lcdStr, st_test_single_pressure);
					lcd_printf(1, 4, lcdStr);
					break;				
				case state_prep_test:
					load_string_test_info(lcdStr, st_test_single_prep_test);
					lcd_printf(1, 3, lcdStr);
					load_string_test_info(lcdStr, st_test_single_pressure);
					lcd_printf(1, 4, lcdStr);
					break;				
				case state_testing:
					load_string_test_info(lcdStr, st_test_single_testing);
					lcd_printf(1, 3, lcdStr);
					load_string_test_info(lcdStr, st_test_single_pressure);
					lcd_printf(1, 4, lcdStr);
					break;				
				case state_complete:
					if (test_result1)
						load_string_test_info(lcdStr, st_test_single_pass);
					else
						load_string_test_info(lcdStr, st_test_single_fail);
					lcd_printf(1, 3, lcdStr);

					load_string_test_info(lcdStr, st_test_single_pressure);
					lcd_printf(1, 4, lcdStr);
					break;				
			}
		}
		else
		{
			read_dual_pressures(&pressure[0], &pressure[1]);
			check_test_state();

			load_string_tests_info(lcdStr, st_test_dual_units);
			load_string(st_load_units, units, tempStr);
			fix_string_str(lcdStr, tempStr);
			lcd_printf(1, 1, lcdStr);

			load_string_tests_info(lcdStr, st_test_dual_suits);
			lcd_printf(1, 2, lcdStr);

			load_string_tests_info(lcdStr, st_test_dual_info);					

			//test info channel 1
			switch (test_state[0])
			{
				case state_idle:
					load_string(st_load_tests_info, st_test_dual_idle, tempStr);
					fix_string_str(lcdStr, tempStr);
					break;
				case state_inflate_fast:
					load_string(st_load_tests_info, st_test_dual_inflate_fast, tempStr);
					fix_string_str(lcdStr, tempStr);
					break;				
				case state_inflate_slow:
					load_string(st_load_tests_info, st_test_dual_inflate_slow, tempStr);
					fix_string_str(lcdStr, tempStr);
					break;				
				case state_stabilising:
					load_string(st_load_tests_info, st_test_dual_stabilise, tempStr);
					fix_string_long(tempStr, stabilise[0], 3, false);
					fix_string_str(lcdStr, tempStr);
					break;				
				case state_prep_test:
					load_string(st_load_tests_info, st_test_dual_prep_test, tempStr);
					fix_string_str(lcdStr, tempStr);
					break;				
				case state_testing:
					load_string(st_load_tests_info, st_test_dual_samples, tempStr);
					fix_string_long(lcdStr, sample_num[0], 3, true);
					strcpy(tempStr, "of#");
					fix_string_str(lcdStr, tempStr);
					fix_string_long(lcdStr, test_standard_info.total_samples, 3, false);
					break;				
				case state_complete:
					if (test_result1)
						load_string(st_load_tests_info, st_test_dual_passed, tempStr);
					else
						load_string(st_load_tests_info, st_test_dual_failed, tempStr);

					fix_string_str(lcdStr, tempStr);
					break;				
			}

			//test info channel 2
			switch (test_state[1])
			{
				case state_idle:
					load_string(st_load_tests_info, st_test_dual_idle, tempStr);
					fix_string_str(lcdStr, tempStr);
					break;
				case state_inflate_fast:
					load_string(st_load_tests_info, st_test_dual_inflate_fast, tempStr);
					fix_string_str(lcdStr, tempStr);
					break;				
				case state_inflate_slow:
					load_string(st_load_tests_info, st_test_dual_inflate_slow, tempStr);
					fix_string_str(lcdStr, tempStr);
					break;				
				case state_stabilising:
					load_string(st_load_tests_info, st_test_dual_stabilise, tempStr);
					fix_string_long(tempStr, stabilise[1], 3, false);
					fix_string_str(lcdStr, tempStr);
					break;				
				case state_prep_test:
					load_string(st_load_tests_info, st_test_dual_prep_test, tempStr);
					fix_string_str(lcdStr, tempStr);
					break;				
				case state_testing:
					load_string(st_load_tests_info, st_test_dual_samples, tempStr);
					fix_string_long(lcdStr, sample_num[1], 3, true);
					strcpy(tempStr, "of#");
					fix_string_str(lcdStr, tempStr);
					fix_string_long(lcdStr, test_standard_info.total_samples, 3, false);
					break;				
				case state_complete:
					if (test_result2)
						load_string(st_load_tests_info, st_test_dual_passed, tempStr);
					else
						load_string(st_load_tests_info, st_test_dual_failed, tempStr);

					fix_string_str(lcdStr, tempStr);
					break;				
			}
					
			lcd_printf(1, 3, lcdStr);					

			load_string_tests_info(lcdStr, st_test_dual_info);					
			
			//test pressure channel 1
			switch (test_state[0])
			{
				case state_complete:
					strcpy(tempStr, "P:#");
					fix_string_str(lcdStr, tempStr);
					fix_string_pressure(lcdStr, final_pressure[0], false);
					break;		
				default:
					strcpy(tempStr, "P:#");
					fix_string_str(lcdStr, tempStr);
					fix_string_pressure(lcdStr, pressure[0], false);
					break;
			}

			//test pressure channel 2
			switch (test_state[1])
			{
				case state_complete:
					strcpy(tempStr, "P:#");
					fix_string_str(lcdStr, tempStr);
					fix_string_pressure(lcdStr, final_pressure[1], false);
					break;		
				default:
					strcpy(tempStr, "P:#");
					fix_string_str(lcdStr, tempStr);
					fix_string_pressure(lcdStr, pressure[1], false);
					break;
			}

			lcd_printf(1, 4, lcdStr);
		}
		tempStr[0] = CMD_TIMER_TICK;
		rs232_write_bytes(&tempStr[0], 1);
	}
}

//----------------------------------------------------------------------------------
void check_test_state(void)
//----------------------------------------------------------------------------------
{
	switch(test_state[0])
	{
		case state_begin_test:
			test_result1 = false;
			set_solenoid(sol_vent1, true);

			if (pascal_pressure[0] >= test_standard_info.slow_fill_pa)
			//We are already above the slow fill pressure so go straight to stabilise
			{
				test_state[0] = state_stabilising;
				stabilise[0] = test_standard_info.stabilise_time_seconds;
				dwell_time[0] = test_standard_info.stabilise_dwell_seconds;
			}
			else if (pascal_pressure[0] >= test_standard_info.fast_fill_pa)
			//We are already above the fast fill pressure so go straight to slow fill
			{
				test_state[0] = state_inflate_slow;
				set_solenoid(sol_slow1, true);
			}
			else
			//We are below both pressures so go to fast fill
			{
				test_state[0] = state_inflate_fast;
				set_solenoid(sol_fast1, true);
			}
			break;
		case state_inflate_fast:
			if (pascal_pressure[0] >= test_standard_info.fast_fill_pa)
			{
				test_state[0] = state_inflate_slow;
				set_solenoid(sol_fast1, false);
				set_solenoid(sol_slow1, true);
			}
			else
			{
				inflate_duration[0] += 1;

				if (pascal_pressure[0] < 15)
				{
					//we're less than 15 Pa so check how long we have been like this
					//the pressure transducer may have fallen off the board and we
					//do not want to keep inflating too long in case we blow it up!
					if (inflate_duration[0] > 150)
					{
						//that's too long, we need to abort
						test_state[0] = state_complete;
						set_solenoid(sol_fast1, false);
						set_solenoid(sol_vent1, false);
						test_result1 = false;
						final_pressure[0] = 0;
					}
				}
			}
			break;				
		case state_inflate_slow:
			if (pascal_pressure[0] >= test_standard_info.slow_fill_pa)
			{
				test_state[0] = state_stabilising;
				set_solenoid(sol_slow1, false);
				stabilise[0] = test_standard_info.stabilise_time_seconds;
				dwell_time[0] = test_standard_info.stabilise_dwell_seconds;
			}
			break;				
		case state_stabilising:
			stabilise[0] -= 1;
			dwell_time[0] -= 1;

			if ((stabilise[0] == 0) || ((dwell_time[0] != 0) && (dwell_time[0] <= (test_standard_info.stabilise_dwell_seconds / 2))))
			{
				set_solenoid(sol_slow1, false);
				set_solenoid(sol_vent1, true);
				if (stabilise[0] == 0)
				{
					dwell_time[0] = test_standard_info.test_deflate_pause_seconds; //add this
					test_state[0] = state_prep_test;
					set_solenoid(sol_vent1, false);	//start venting to get down to test pressure
				}
			}
			else
			{
				if (dwell_time[0] == 0)
				{
					dwell_time[0] = test_standard_info.stabilise_dwell_seconds;
					if (pascal_pressure[0] >= (test_standard_info.stabilise_pa + test_standard_info.stabilise_hysteresis_pa))
					{
						set_solenoid(sol_vent1, false);
						set_solenoid(sol_slow1, false);
					}
					else if (pascal_pressure[0] <= (test_standard_info.stabilise_pa - test_standard_info.stabilise_hysteresis_pa))
					{
						set_solenoid(sol_vent1, true);
						set_solenoid(sol_slow1, true);
					}
					else
					{
						set_solenoid(sol_vent1, true);
						set_solenoid(sol_slow1, false);
					}
				}	
			}
			break;				
		case state_prep_test:
			if (pascal_pressure[0] <= test_standard_info.test_pressure_pa)
			{
				set_solenoid(sol_vent1, true); //stop venting
				test_state[0] = state_testing;
			}
			else
			{
				if (dwell_time[0] > 0)
				{
					if (dwell_time[0] == test_standard_info.test_deflate_pause_seconds)
					{
						set_solenoid(sol_vent1, true); //stop venting
					}
					dwell_time[0]--;
				}
				else
				{
					dwell_time[0] = test_standard_info.test_deflate_pause_seconds;
					
					if (test_standard_info.test_deflate_pause_seconds > 0)
					{
						set_solenoid(sol_vent1, false); //start venting
					}
				}
			}
			break;
		case state_testing:
			sample_num[0] += 1;

			if (sample_num[0] == test_standard_info.total_samples)
			{
				test_state[0] = state_complete;
				set_solenoid(sol_vent1, false);
				test_result1 = pascal_pressure[0] > test_standard_info.limit;
				final_pressure[0] = pressure[0];
			}
			break;				
		case state_complete:
			//do nothing
			break;				
	}

	if (dual_channel)
	{
		switch(test_state[1])
		{
			case state_begin_test:
				test_result1 = false;
				set_solenoid(sol_vent2, true);

				if (pascal_pressure[1] >= test_standard_info.slow_fill_pa)
				//We are already above the slow fill pressure so go straight to stabilise
				{
					test_state[1] = state_stabilising;
					stabilise[1] = test_standard_info.stabilise_time_seconds;
					dwell_time[1] = test_standard_info.stabilise_dwell_seconds;
				}
				else if (pascal_pressure[1] >= test_standard_info.fast_fill_pa)
				//We are already above the fast fill pressure so go straight to slow fill
				{
					test_state[1] = state_inflate_slow;
					set_solenoid(sol_slow2, true);
				}
				else
				//We are below both pressures so go to fast fill
				{
					test_state[1] = state_inflate_fast;
					set_solenoid(sol_fast2, true);
				}
				break;
			case state_inflate_fast:
				if (pascal_pressure[1] >= test_standard_info.fast_fill_pa)
				{
					test_state[1] = state_inflate_slow;
					set_solenoid(sol_fast2, false);
					set_solenoid(sol_slow2, true);
				}
				else
				{
					inflate_duration[1] += 1;

					if (pascal_pressure[1] < 15)
					{
						//we're less than 15 Pa so check how long we have been like this
						//the pressure transducer may have fallen off the board and we
						//do not want to keep inflating too long in case we blow it up!
						if (inflate_duration[1] > 150)
						{
							//that's too long, we need to abort
							test_state[1] = state_complete;
							set_solenoid(sol_fast2, false);
							set_solenoid(sol_vent2, false);
							test_result2 = false;
							final_pressure[1] = 0;
						}
					}
				}
				break;				
			case state_inflate_slow:
				if (pascal_pressure[1] >= test_standard_info.slow_fill_pa)
				{
					test_state[1] = state_stabilising;
					set_solenoid(sol_slow2, false);
					stabilise[1] = test_standard_info.stabilise_time_seconds;
					dwell_time[1] = test_standard_info.stabilise_dwell_seconds;
				}
				break;				
			case state_stabilising:
				stabilise[1] -= 1;
				dwell_time[1] -= 1;

				if ((stabilise[1] == 0) || ((dwell_time[1] != 0) && (dwell_time[1] <= (test_standard_info.stabilise_dwell_seconds / 2))))
				{
					set_solenoid(sol_slow2, false);
					set_solenoid(sol_vent2, true);
					if (stabilise[1] == 0)
					{
						dwell_time[1] = test_standard_info.test_deflate_pause_seconds; //add this
						test_state[1] = state_prep_test;
						set_solenoid(sol_vent2, false);	//start venting to get down to test pressure
					}
				}
				else
				{
					if (dwell_time[1] == 0)
					{
						dwell_time[1] = test_standard_info.stabilise_dwell_seconds;
						if (pascal_pressure[1] >= (test_standard_info.stabilise_pa + test_standard_info.stabilise_hysteresis_pa))
						{
							set_solenoid(sol_vent2, false);
							set_solenoid(sol_slow2, false);
						}
						else if (pascal_pressure[1] <= (test_standard_info.stabilise_pa - test_standard_info.stabilise_hysteresis_pa))
						{
							set_solenoid(sol_vent2, true);
							set_solenoid(sol_slow2, true);
						}
						else
						{
							set_solenoid(sol_vent2, true);
							set_solenoid(sol_slow2, false);
						}
					}	
				}
				break;				
			case state_prep_test:
				if (pascal_pressure[1] <= test_standard_info.test_pressure_pa)
				{
					set_solenoid(sol_vent2, true);
					test_state[1] = state_testing;
				}
				else
				{
					if (dwell_time[1] > 0)
					{
						if (dwell_time[1] == test_standard_info.test_deflate_pause_seconds)
						{
							set_solenoid(sol_vent2, true); //stop venting
						}
						dwell_time[1]--;
					}
					else
					{
						dwell_time[1] = test_standard_info.test_deflate_pause_seconds;
						
						if (test_standard_info.test_deflate_pause_seconds > 0)
						{
							set_solenoid(sol_vent2, false); //start venting
						}
					}
				}
				break;
			case state_testing:
				sample_num[1] += 1;

				if (sample_num[1] == test_standard_info.total_samples)
				{
					test_state[1] = state_complete;
					set_solenoid(sol_vent2, false);
					test_result2 = pascal_pressure[1] > test_standard_info.limit;
					final_pressure[1] = pressure[1];
				}
				break;				
			case state_complete:
				//do nothing
				break;				
		}

		if ((test_state[0] == state_complete) && (test_state[1] == state_complete))
		{
			testing = false;
		}
	}
	else
	{
		testing = test_state[0] != state_complete;
	}
}

//----------------------------------------------------------------------------------
void reset_solenoids(unsigned int8 channel)
//----------------------------------------------------------------------------------
{
	if (channel == 0)
	{
		set_solenoid(sol_fast1, false);
		set_solenoid(sol_slow1, false);
		set_solenoid(sol_vent1, false);
	}
	else
	{
		set_solenoid(sol_fast2, false);
		set_solenoid(sol_slow2, false);
		set_solenoid(sol_vent2, false);
	}
}

//----------------------------------------------------------------------------------
void set_solenoid(solenoids solenoid, bool active)
//----------------------------------------------------------------------------------
{
	if (active)
	{
		switch (solenoid)
		{
			case sol_fast1:
				if (!dual_channel)
				{
					output_high(FAST_FILL1A);
				}
				else
				{
					output_high(FAST_FILL1B);
				}
				break;
			case sol_slow1:
				if (!dual_channel)
				{
					output_high(SLOW_FILL1A);
				}
				else
				{
					output_high(SLOW_FILL1B);
				}
				break;
			case sol_vent1:
				if (!dual_channel)
				{
					output_high(VENT1A);
				}
				else
				{
					output_high(VENT1B);
				}
				break;
			case sol_fast2:
				output_high(FAST_FILL2);
				break;
			case sol_slow2:
				output_high(SLOW_FILL2);
				break;
			case sol_vent2:
				output_high(VENT2);
				break;
		}
	}
	else
	{
		switch (solenoid)
		{
			case sol_fast1:
				if (!dual_channel)
				{
					output_low(FAST_FILL1A);
				}
				else
				{
					output_low(FAST_FILL1B);
				}
				break;
			case sol_slow1:
				if (!dual_channel)
				{
					output_low(SLOW_FILL1A);
				}
				else
				{
					output_low(SLOW_FILL1B);
				}
				break;
			case sol_vent1:
				if (!dual_channel)
				{
					output_low(VENT1A);
				}
				else
				{
					output_low(VENT1B);
				}
				break;
			case sol_fast2:
				output_low(FAST_FILL2);
				break;
			case sol_slow2:
				output_low(SLOW_FILL2);
				break;
			case sol_vent2:
				output_low(VENT2);
				break;
		}
	}
}

// =============================================================
#endif